@@ -86,13 +86,18 @@ rd_Z(Uint *n, int w, ftnlen len)
 		if (!--w)
 			return 0;
 		t += i;
-		}
+	}
+	
+#pragma GCC diagnostic ignored "-Wparentheses"
+	
 	do {
 		*t = hex[*s0 & 0xff]-1 << 4 | hex[s0[1] & 0xff]-1;
 		t += i;
 		s0 += 2;
 		}
 		while(--w);
+#pragma GCC diagnostic warning "-Wparentheses"		
+		
 	return 0;
 	}
 
